/*******************************************************************************
**
**  Mouse Evaluation & Test Program
**  (c) Andrea Richards 1996,7
**
** This C++ program demonstrates some of the uses of the vsa256 Event Handler
** It has been written using Borland BC4.52 and tested on BC5.01
**
** Email: 100102.3262@compuserve.com
**
*******************************************************************************/

#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "vsamouse.h"
#include "vsa_font.h"       /* Required to support VSA256 Graphics Library  */
                            /*   Include this header just the once per app */

#ifndef _MSC_VER
/*.....                  This is for Borland C Only !                  .....*/
#pragma warn -wsig
extern unsigned _stklen = 40000L;
#pragma warn .wsig
#endif

void shaded_lut(void);

char MouseHeader[] = "MouseX    MouseY   MouseMsg";
char *MouseMessages[] = {
      "WM_MOUSEMOVE",
      "WM_LBUTTONDOWN",
      "WM_LBUTTONUP",
      "WM_LBUTTONDBLCLK",
      "WM_RBUTTONDOWN",
      "WM_RBUTTONUP",
      "WM_RBUTTONDBLCLK",
      "WM_MBUTTONDOWN",
      "WM_MBUTTONUP",
      "WM_MBUTTONDBLCLK" };

int   ShadMsg[] = { -255, 0, 0, 0, 0, 0, 0, 0 };
int   NumMsgs = 8;
float MsgFont[] = { 1.5, 1.5, 0.95, 0.95, 0.9, 0.9, 0.9, 0.9 };
int   MsgColor[] = { 223, 31, 95, 90, 28, 28, 28,255};

char *WindowMessage[] = {
  "VSA256 Enhanced Keyboard and Mouse Handler",
  "VSA256 Enhanced Keyboard and Mouse Handler",
  "(c) Andrea Richards 1996",
  "VSA256 is (c) Spuro Gumas 1996",
  " ",
  "Mouse Messages are shown in the upper region together with nemonic",
  "When the mouse cursor is moved over this lower window border, the cursor will change.",
  "Press Escape to exit."
  };





void main()
{
  int x,y,i,vesa_mode = 0x105;
  long msg;
  char tbuff[200];
  RECT ma;             // Main area rectangle
  RECT ia;             // Information area
  RECT cr;             // Cursor testing area

  BOOL run_demo = TRUE;
  EventHandler *ev;

  // Try and obtain a graphics mode, report if an error
	if((i = vsa_init(vesa_mode)) != 0)	{
  	printf("Error Initializing Requested Video Mode!\n");
		if(i==1) printf("  - Did You Load Correct VESA Driver (TSR) ??\n");
		if(i==2) printf("  - VESA BIOS Extensions (Driver) Not Loaded !!\n");
		if(i==3) printf("  - Requested Video Mode Not Supported by this Card!\n");
		if(i==4) printf("  - Mode Not an SVGA Mode Supported by this Card!\n");
		if(i==5) printf("  - VESA Driver Not Returning Mode Information!\n");
		exit(0);
		}

  // Generate lookup table of colours
  shaded_lut();

  // Create an EventHandler  with a max message buffer size of 200
  ev = new EventHandler(200);

  // Performing a reset will test if a mouse is available and reset everything
  // to know values
  if (!ev->reset()) {
    vsa_init(0x03);     // Reset to a text mode
    printf("Unable to locate a mouse driver\n");
    printf("Press a key to continue...\n");
    getch();
    exit(0);
    }

  // Setup a few areas onto the screen
  // This is the main Window Area and we can force the mouse to limit its
  // movements to this window only

  // The MouseHandler holds the main screen area, so we can call the get_status
  // to obtain this information. Shrink down the zone area to restrict the
  // mouse movement.
  MOUSE_STAT stat;
  int pointer_width, pointer_height;
  ev->MouseHandler::get_status(&stat);
  ev->get_mouse_size(&pointer_width, &pointer_height);
  ma.left   = stat.Zone.left + 150;
  ma.top    = stat.Zone.top + 150;
  ma.right  = stat.Zone.right - 150;
  ma.bottom = stat.Zone.bottom - 300;

  // Uncomment the next line to restrict the mouse to the Region defined
  // as ma
  // ev->set_zone(ma);

  // Highlight the main area
  vsa_set_color(63) ;
  vsa_move_to(ma.left, ma.top);
  vsa_rect(ma.right, ma.bottom);

  // Setup the information area
  ia.top    = ma.top + 20;
  ia.left   = ma.left + 20;
  ia.right  = ma.right - 20;
  ia.bottom = ia.top + 50;

  // Highlight the info area and include the cursor width & height;
  vsa_set_color(127) ;
  vsa_move_to(ia.left, ia.top);
  vsa_rect(ia.right, ia.bottom);

  // Create the area where the mouse will turn into various cursors if the
  // mouse enters a particular region

  cr.top    = ia.bottom + 20;
  cr.bottom = ma.bottom - 20;
  cr.right  = ma.right - 20;
  cr.left   = ma.left + 20;

  // Highlight the info area and include the cursor width & height;
  // Draw a MS Windows style Frame, nothing elaborate here.

  int border_width = 3;
  int delta_corner = 20;

  vsa_set_color(22) ;
  vsa_move_to(cr.left, cr.top);
  vsa_rect(cr.right, cr.bottom);
  vsa_move_to(cr.left+border_width, cr.top+border_width);
  vsa_rect(cr.right-border_width, cr.bottom-border_width);

  vsa_move_to(cr.left, cr.top+delta_corner);
  vsa_line_to(cr.left+border_width,cr.top+delta_corner);
  vsa_move_to(cr.right-border_width, cr.top+delta_corner);
  vsa_line_to(cr.right,cr.top+delta_corner);

  vsa_move_to(cr.left, cr.bottom-delta_corner);
  vsa_line_to(cr.left+border_width,cr.bottom-delta_corner);
  vsa_move_to(cr.right-border_width, cr.bottom-delta_corner);
  vsa_line_to(cr.right,cr.bottom-delta_corner);

  vsa_move_to(cr.left+delta_corner, cr.top);
  vsa_line_to(cr.left+delta_corner, cr.top+border_width);
  vsa_move_to(cr.right-delta_corner, cr.top);
  vsa_line_to(cr.right-delta_corner, cr.top+border_width);

  vsa_move_to(cr.left+delta_corner, cr.bottom-border_width);
  vsa_line_to(cr.left+delta_corner, cr.bottom);
  vsa_move_to(cr.right-delta_corner, cr.bottom-border_width);
  vsa_line_to(cr.right-delta_corner, cr.bottom);


  // The mouse is already being displayed at this point so we must
  // hide it before moving it to a new position.
  ev->hide_cursor();
  ev->set_cursor(ia.left+((ia.right-ia.left)/2),
                ia.top +((ia.bottom-ia.top)/2) );
  ev->show_cursor();

  // Write out some information
  vsa_write_string(ia.left+5, ia.top+5,223,MouseHeader);

  // Cache a few coordinates ready for printing out mouse info
  int mouseinfoX, mouseinfoY, mouseinfoW, mouseinfoH;
  mouseinfoX = ia.left+5;
  mouseinfoY = ia.top+5 + YCharSize;
  mouseinfoW = mouseinfoX+(5+strlen(MouseHeader)+strlen(MouseMessages[3]))*XCharSize;
  mouseinfoH = mouseinfoY+YCharSize;

  // Write some info onto the screen
  int xp = 10+ cr.left + XCharSize;
  for (i = 0; i < NumMsgs; i++ )  {
    vsa_set_text_scale(MsgFont[i], MsgFont[i]);
    vsa_write_string(xp, cr.top+YCharSize + (i*(YCharSize+3)),MsgColor[i], WindowMessage[i]);
    if (ShadMsg[i]) {
      vsa_write_string(xp+1, cr.top+YCharSize + (i*(YCharSize+3)),MsgColor[i+1], WindowMessage[i+1]);
      i++;
      }
    }

  // *************************************************************************
  // Now enter the main Event Loop
  // *************************************************************************

  while (run_demo)  {
    // If No messages are pending then nothing to do
    if (!ev->GetMessage(&x, &y, &msg)) continue;      // Message available ?

    int MouseMsg = HIWORD(msg);                       // Mask out Mouse Messages
    if (MouseMsg >= WM_MOUSEFIRST && MouseMsg <= WM_MOUSELAST)  { // Mouse !!

      ev->hide_cursor();                // Hide the cursor whilst drawing
                                        // It would be better to see if the
                                        // cursor was actually within the drawing area
                                        // and only hide it if necessarry

      // Clear the mouse status info area
      vsa_set_color(0);
      vsa_move_to(mouseinfoX ,mouseinfoY);
      vsa_rect_fill(mouseinfoW, mouseinfoH);
      // Generate some information for the demo
      sprintf(tbuff," %04x       %04X      %08lx     %s",x, y, msg, MouseMessages[MouseMsg-WM_MOUSEFIRST]);
      vsa_write_string(mouseinfoX,mouseinfoY,31,tbuff);
      ev->show_cursor();                             // show the cursor again

      switch (MouseMsg)  {

        case WM_LBUTTONDBLCLK:
        case WM_RBUTTONDBLCLK:
        case WM_MBUTTONDBLCLK:    // Disabled in Shareware Version
          sound(1800);
          delay(80);
          nosound();
          break;

        case WM_MOUSEMOVE:
          // In this example, we will demonstrate how to change the cursor
          // The cursor shape will depend on the region where the mouse is
          // located;
          int sizer = ev->check_for_sizer(cr, border_width, delta_corner);
          if (sizer != -1) ev->set_cursor_type(sizer);
          else ev->set_cursor_type(WM_MOUSE_ARROW);
          break;
      } // End switch
    }

    // We can test for Keyboard events too as this IS an event handler
    // Look for a key pressed and if its the escape key leave the demo
    // scan codes are held in the x pos
    // modifiers are y pos
    // There may be a better way to get the data back to the application
    if (MouseMsg == WM_KEYPRESS)  {
      int keypressed = x & 0xFF;
      if (keypressed == ESC) run_demo = FALSE;
    }
  }   // End while

  delete ev;
	vsa_init(0x3);      // Back to text mode
  return;
}


// Taken from Spyro's woking example
/*.............................. SHADED_LUT.C ............. 2-25-96 ........*/
/*  This routine generates a lookup table holding 8 primary colors with 32  */
/*  shades of "gray" for each.                                              */
/*                                                                          */
/*           COLOR   |   INDEX RANGE                                        */
/*           -----------------------                                        */
/*            Gray   |     0-31                                             */
/*            Red    |    32-63                                             */
/*            Orange |    64-95                                             */
/*            Yellow |    96-127                                            */
/*            Green  |   128-159                                            */
/*            Auqa   |   160-191                                            */
/*            Blue   |   192-223                                            */
/*            Purple |   224-255                                            */
/*..........................................................................*/
void shaded_lut(void)
{
	int i,j;
	unsigned char color_array[768];
	float r[8]={1.0,1.0,0.917,1.0,0.0,0.0,0.0,1.0};
	float g[8]={1.0,0.0,0.436,1.0,1.0,1.0,0.0,0.0};
	float b[8]={1.0,0.0,0.000,0.0,0.0,1.0,1.0,1.0};
	float k;
	for(j=0;j<8;j++)
		{
			for(i=0;i<32;i++)
				{
					if(i < 12) k = pow((i/31.0),1.0);
					else       k = pow((11/31.0),1.0) + 0.645*pow(((i-9.0)/(31.0-9.0)),3.0);
					color_array[3*i+96*j+0]= (int)(k*63*r[j] + 0.5);
					color_array[3*i+96*j+1]= (int)(k*63*g[j] + 0.5);
					color_array[3*i+96*j+2]= (int)(k*63*b[j] + 0.5);
					i=i;
				}
		}
	vsa_write_color_block(0,256,color_array);
	return;
}                                   /*.....       End shaded_lut       .....*/



