#ifndef __vsamouse_h__
#define __vsamouse_h__

#include <bios.h>
#include <dos.h>
#include <mem.h>
#include "vsa.h"            /* Required to support VSA256 Graphics Library  */


// If the class libraries are being used at all then windows.h will be pulled
// in and then there will be multiple declarations.
#ifndef __WINDOWS_H
typedef int                       BOOL;
typedef unsigned char             BYTE;
typedef unsigned int              WORD;
typedef unsigned long             DWORD;
typedef long                      LONG;
typedef void far *                HANDLE;

typedef struct tagPOINT {
  int x;
  int y;
  } POINT;

typedef struct tagRECT  {
  int top;
  int left;
  int bottom;
  int right;
  } RECT;

/* Mouse input messages */
#define WM_MOUSEMOVE        0x0200
#define WM_LBUTTONDOWN      0x0201
#define WM_LBUTTONUP        0x0202
#define WM_LBUTTONDBLCLK    0x0203
#define WM_RBUTTONDOWN      0x0204
#define WM_RBUTTONUP        0x0205
#define WM_RBUTTONDBLCLK    0x0206
#define WM_MBUTTONDOWN      0x0207
#define WM_MBUTTONUP        0x0208
#define WM_MBUTTONDBLCLK    0x0209
#define WM_MOUSEFIRST       WM_MOUSEMOVE
#define WM_MOUSELAST        WM_MBUTTONDBLCLK


#define NULL                0L
#define FALSE               0
#define TRUE                1

#define max(a, b)  (((a) > (b)) ? (a) : (b))
#define min(a, b)  (((a) < (b)) ? (a) : (b))

#endif  // End of Windows definitions and macro's

// Mouse Button Constants
#define WM_LBUTTON            0x01F0
#define WM_RBUTTON            0x01F1
#define WM_MBUTTON            0x01F2

// Mouse Cursor Constants
#define WM_MOUSE_ARROW        0x020B
#define WM_MOUSE_HOURGLASS    0x020C
#define WM_MOUSE_IBEAM        0x020D
#define WM_MOUSE_USER         0x020E
#define WM_MOUSE_VERT_SIZER   0x0210
#define WM_MOUSE_HORIZ_SIZER  0x0211
#define WM_MOUSE_MOVER        0x0212
#define WM_MOUSE_TRBLD_SIZER  0x0213
#define WM_MOUSE_TLBRD_SIZER  0x0214


// A few common macros
#ifndef LOBYTE
#define LOBYTE(w)           ((BYTE)(w))
#endif
#ifndef HIBYTE
#define HIBYTE(w)           ((BYTE)((WORD)(w) >> 8))
#endif
#ifndef LOWORD
#define LOWORD(l)           ((WORD)(l))
#endif
#ifndef HIWORD
#define HIWORD(l)           ((WORD)((DWORD)(l) >> 16))
#endif
#ifndef MAKELONG
#define MAKELONG(low, high) ((LONG)(((WORD)(low)) | (((DWORD)((WORD)(high))) << 16)))
#endif
#ifndef MAKEWORD
#define MAKEWORD(low, high) ((WORD)(((BYTE)(low)) | (((WORD) ((BYTE)(high))) << 8)))
#endif

// Keyboard Handler KeyCodes for the PC / AT
#define WM_KEYPRESS 0x0220
#define F1          0x003b
#define F2          0x003c
#define F3          0x003d
#define F4          0x003e
#define F5          0x003f
#define F6          0x0040
#define F7          0x0041
#define F8          0x0042
#define F9          0x0043
#define F10         0x0044
#define F11         0x0098
#define F12         0x0099

#define SHIFT_F1    0x0054
#define SHIFT_F2    0x0055
#define SHIFT_F3    0x0056
#define SHIFT_F4    0x0057
#define SHIFT_F5    0x0058
#define SHIFT_F6    0x0059
#define SHIFT_F7    0x005a
#define SHIFT_F8    0x005b
#define SHIFT_F9    0x005c
#define SHIFT_F10   0x005d
#define SHIFT_F11   0x00a2
#define SHIFT_F12   0x00a3
#define SHIFT_I     0x000f

#define ALT_F1      0x0068
#define ALT_F2      0x0069
#define ALT_F3      0x006a
#define ALT_F4      0x006b
#define ALT_F5      0x006c
#define ALT_F6      0x006d
#define ALT_F7      0x006e
#define ALT_F8      0x006f
#define ALT_F9      0x0070
#define ALT_F10     0x0071
#define ALT_F11     0x00b6
#define ALT_F12     0x00b7

#define CTRL_F1     0x005e
#define CTRL_F2     0x005f
#define CTRL_F3     0x0060
#define CTRL_F4     0x0061
#define CTRL_F5     0x0062
#define CTRL_F6     0x0063
#define CTRL_F7     0x0064
#define CTRL_F8     0x0065
#define CTRL_F9     0x0066
#define CTRL_F10    0x0067
#define CTRL_F11    0x00ac
#define CTRL_F12    0x00ad

#define UP          0x0048
#define DOWN        0x0050
#define PGUP        0x0049
#define PGDN        0x0051
#define LEFT        0x004b
#define RIGHT       0x004d
#define HOME        0x0047
#define END         0x004f
#define DEL         0x0053
#define INS         0x0052

/* Non-scan code keys, flagged as NULL scan-codes  */
#define CTRL_H      0x0008
#define CTRL_M      0x000d
#define CTRL_J      0x000a
#define CTRL_L      0x000c
#define CTRL_C      0x0003
#define CTRL_I      0x0009

#define ALT_A       0x1E00
#define ALT_B       0x3000
#define ALT_C       0x2E00
#define ALT_D       0x2000
#define ALT_E       0x1200
#define ALT_F       0x2100
#define ALT_G       0x2200
#define ALT_H       0x2300
#define ALT_I       0x1700
#define ALT_J       0x2400
#define ALT_K       0x2500
#define ALT_L       0x2600
#define ALT_M       0x3200
#define ALT_N       0x3100
#define ALT_O       0x1800
#define ALT_P       0x1900
#define ALT_Q       0x1000
#define ALT_R       0x1300
#define ALT_S       0x1F00
#define ALT_T       0x1400
#define ALT_U       0x1600
#define ALT_V       0x2F00
#define ALT_W       0x1100
#define ALT_X       0x2D00
#define ALT_Y       0x1500
#define ALT_Z       0x2C00
#define SHIFT_TAB   0x0F00


// Enhanced Key Codes which can be AND'd with the LOWORD of the msg in the
// message que. This will help with say ALT + WM_RBUTTONDOWN etc

#define SYS_REQ       0x8000
#define CAPS_LOCK     0x4000
#define NUM_LOCK      0x2000
#define SCROLL_LOCK   0x1000
#define RIGHT_ALT     0x0800
#define RIGHT_CTRL    0x0400
#define LEFT_ALT      0x0200
#define LEFT_CTRL     0x0100

// Standard Key Codes
#define ESC           0x001B
#define CR            CTRL_M
#define LF            CTRL_J
#define TAB           CTRL_I
#define PLUS          '+'
#define MINUS         '-'

#define INSERT_ON     0x0080
#define CAPS_ON       0x0040
#define NUMLOCK_ON    0x0020
#define SCROLLLOCK_ON 0x0010
#define ALT_PRESSED   0x0008
#define CTRL_PRESSED  0x0004
#define LEFT_SHIFT    0x0002
#define RIGHT_SHIFT   0x0001

// End of Keyboard Scan Codes Phew !

class MouseHandler;
class EventHandler;
class CaretHandler;

typedef struct tagCURSOR {
  int width;            // Cursor Image Width (max 32)
  int height;           // Cursor Image Height (max 32)
  int cursortype;       // Contains the current cursor e.g. WM_MOUSE_ARROW
  POINT hotspot;        // Cursor Hotspot
  } CURSOR;

typedef struct tagCARET {
  int width;            // Caret Image Width
  int height;           // Caret Image Height
  int cursortype;       // Contains the current cursor e.g. WM_MOUSE_ARROW
  POINT coords;         // Caret coordinates
  RECT Zone;            // Restricted Caret region
  int text_width;       // == XCharSize for this object
  int text_height;      // == YCharSize for this object
  int line_spacing;     // distance in pixels between lines of text
  int tab_stops;        // for simple tabbing
  } CARET;

typedef struct tagMOUSE_STAT {
  BOOL MouseAvailable;  // TRUE if a mouse driver was located
  int xpos,             // XPos of the mouse (not the mouse hot spot)
      ypos;             // YPos of the mouse (not the mouse hot spot)
  BOOL lb_stat,         // TRUE whilst the Left Button is being held down
       mb_stat,         // TRUE whilst the Middle Button is being held down
       rb_stat;         // TRUE whilst the Right Button is being held down
  RECT Zone;            // Holds the restricted mouse region
  BOOL CursorDisplay;   // TRUE if the mouse is on the screen
  int  NumColors;       // Number of colors available for the selected mode
  } MOUSE_STAT;



// Circular FIFO Buffer (First in First Out)
// This is a base class which holds event messages
class Circ_FIFO {
  public:
    enum { FIFO_EMPTY = 2, FIFO_FULL, FIFO_HASDATA, FIFO_UPDATED};
    Circ_FIFO(int maxsize);
    ~Circ_FIFO();
    virtual short GetMessage(int *x, int *y, long *msg);
    virtual short PostMessage(int x, int y, long msg);
    virtual short PeekMessage(int *x, int *y, long *msg);
  public:
    void  FIFOFlush();
    short IsFIFOEmpty();
    short IsFIFOFull();
    int   GetFIFOEntries() { return NumEntries; };
  private:
    unsigned long *buffer;
    int PutIndex;
    int GetIndex;
    int MaxCount;
    int NumEntries;
};


// Mouse Handler Class based on the FIFO so that mouse messages can be que'd
// Only one object of this class should be used at any one time, or you may end
// up with er, a mess...
class MouseHandler : public Circ_FIFO {

    enum { Service, Show, Hide };
    friend EventHandler;

  public :
    MouseHandler(int MaxBufferSize);
    ~MouseHandler();
    virtual BOOL reset(void);                               // Always call this when instatiating the class
    virtual short GetMessage(int *x, int *y, long *msg);    // from the FIFO
    virtual short PostMessage( int x, int y, long msg);     // To the FIFO
    virtual short PeekMessage( int *x, int *y, long *msg);  // Peeks Last message
    virtual BOOL get_status(void);                          // returns TRUE if we have a mouse
    virtual BOOL get_status(MOUSE_STAT far *stat);          // Checks all mouse status
    virtual void show_cursor();                             // Displays the cursor if its no shown
    virtual void hide_cursor();                             // Hides the cursor if its not hidden

  public:
    void set_display_page(int page);                        // default is set to 1
    unsigned long get_position(void);                       // returns XposYpos as a long or -1 if no mouse
    BOOL get_button_state(int button);                      // returns TRUE if the button is pressed
    BOOL set_cursor(int x, int y);                          // Moves the cursor to x,y
    void set_cursor_type(int c_type);                       // Changes the cursor on the fly
    BOOL set_zoneX(int min, int max);                       // Sets the mouse restricted zone X1 to X2
    BOOL set_zoneY(int min, int max);                       // Sets the mouse restricted zone Y1 to Y2
    BOOL set_zone(RECT rect);                               // Sets the mouse restricted zone X1,Y1,X2,Y2
    BOOL set_dblclk_zone(POINT *rect);                      // Sets the allowed movement before a DBLCLK is sent
    void force_show_cursor();                               // Will always show the cursor regardless
    void new_cursor(CURSOR *curs);                          // User defined Cursor

    // This function will check a rectangle with a border to see if the mouse is sitting
    // within the border of the region. Can be used to change the mouse cursor to a sizer
    // cursor if resizing a region
    int  check_for_sizer(RECT &outer, int border_width, int delta_corner);

    // Internal Cursor Types
    void set_arrow();                     // Arrow Pointer
    void set_hourglass();                 // HourGlass
    void set_ibeam();                     // IBeam

    void set_vert_sizer();                // Windows Type Cursors
    void set_horiz_sizer();
    void set_mover();
    void set_trbldiag_sizer();
    void set_tlbrdiag_sizer();
    void set_user();
    // To set a User Cursor, draw the cursor on the screen, out of sight
    // Then use vsa_get_image(....) into a tempoaray buffer
    // Call set_user_cursor setting the width & height
    // 32 * 32 is the maximum height, width
    // The Cursor will be enabled on calling.
    // if bmp == NULL the image that is held in the buffer will be used
    // otherwise the image pointed to by bmp will be used

    void set_user_cursor(unsigned char huge *bmp, CURSOR *c); // Not available shareware version

    // Returns the width and height of the current mouse image size
    inline void get_mouse_size(int *width, int *height)
      {*width = Current_Cursor.width; *height = Current_Cursor.height; };

    // Returns the type of cursor currently being displayed
    inline short get_cursor_type()
      { return Current_Cursor.cursortype; };

  protected:
    void draw_cursor(short Caller = Service);   // Draws the cursor. Internal usage only

  private:
    // VERY Private to this class, dont interfere with any of these
    CURSOR Current_Cursor;
    CURSOR User_Cursor;
    union  REGS  i_regs, o_regs;
    struct SREGS seg_regs;
    int    bx, cx, dx;
    int    display_count;
    MOUSE_STAT status;
    BOOL  lbutton,rbutton,mbutton;
    BOOL  HaveMouse;
    int   Click_Delay;
    unsigned char huge *Curs_Buff;

    inline int _timer_int(int service, int *cx_reg, int *dx_reg);
    inline int _mouse_int(int service, int *bx_reg, int *cx_reg, int *dx_reg);

};



// Keyboard Handler based on the FIFO, so it has its own buffer for messages
class KeyHandler  : public Circ_FIFO  {

  public:
    KeyHandler(int MaxBufferSize);
    ~KeyHandler() {};
    virtual BOOL reset();
    virtual short get_status();
    virtual short GetMessage(int *x, int *y, long *msg); // from the FIFO
    virtual short PostMessage( int x, int y, long msg);  // To the FIFO
    virtual short PeekMessage( int *x, int *y, long *msg); // Peeks Last message
};


// Caret Handler class. This class will be expanded in the near future.
//
class CaretHandler  : public MouseHandler {
  enum { Service, Show, Hide };
  enum {CARET_INSERT, CARET_OVERWRITE, USER_INSERT, USER_OVERWRITE };
  public:
    short BackgroundColor;    // Currently Unused with Internal Caret Pointers
    short ForegroundColor;    // Set this to a color register for required color
    CaretHandler(int MaxBufferSize);
    ~CaretHandler();
    virtual BOOL reset();
    virtual unsigned long GetCaretPos(void);
    virtual void SetCaretPos(POINT pos);
    virtual short HideCaret();
    virtual short ShowCaret();
    virtual void SetCaretMode(int mode);
    virtual short getCaretMode() { return Caret.cursortype; };
    virtual void HandleKeys(int lowkey, int highkey);
    virtual void SetCaretZone(RECT *zone);
  private:
    void draw_caret(short Caller = Service);
    unsigned char huge *Insert_Bmp;
    unsigned char huge *Overwrite_Bmp;
    unsigned char huge *User_Insert_Bmp;
    unsigned char huge *User_Overwrite_Bmp;
    unsigned char huge *Curs_Buff;
    CARET Caret;
    short CaretDisplay;
    RECT CaretWindow;
};

// Event Handler based upon the mouse and keyboard handler. It combines the mouse
// keyboard, and Caret Messages so that just the one cll to GetMessage is all
// that is required for the main message loop
class EventHandler  : public CaretHandler, KeyHandler
{
  public:
    EventHandler(int MaxBufferSize);
    ~EventHandler();
    virtual short GetMessage(int *x, int *y, long *msg);
    virtual short PostMessage(int x, int y, long msg);
    virtual BOOL reset();
  public:
    void GetMaxScreen(RECT *r);
    BOOL CheckCursor(RECT &rc);
    void InflateRectByCursorDimen(RECT *rc);
};



#endif